/**
 * Filters the noise with a FIR filter, then iterates the filtered data points
 * to find the first inflection point.
 * 
 * Usage:
 * 
 * ./fir plot limit < input_spectrum
 */

#include "common.cpp"

/* filter coefficients */
static double coef[] = {
    -3.7275388e-004, -7.0615375e-004, -1.1928362e-003, -1.6634197e-003,
    -1.8934395e-003, -1.5538280e-003, -2.3313740e-004,  2.5141526e-003,
     7.0960947e-003,  1.3798921e-002,  2.2704052e-002,  3.3622973e-002,
     4.6067541e-002,  5.9266547e-002,  7.2230911e-002,  8.3863216e-002,
     9.3095648e-002,  9.9033274e-002,  1.0108210e-001,  9.9033274e-002,
     9.3095648e-002,  8.3863216e-002,  7.2230911e-002,  5.9266547e-002,
     4.6067541e-002,  3.3622973e-002,  2.2704052e-002,  1.3798921e-002,
     7.0960947e-003,  2.5141526e-003, -2.3313740e-004, -1.5538280e-003,
    -1.8934395e-003, -1.6634197e-003, -1.1928362e-003, -7.0615375e-004,
    -3.7275388e-004
};

/* number of coefficients (order + 1) */
static int n_coef = 37;

/* convolution */
void conv(const Data& f, const double* g, int s2, double* out)
{
    int s1 = f.size();
    int n, m, l = s1 + s2 - 1;
    for(n = 0; n < l; ++n)
    {
        out[n] = 0;
        int min = n - s2 + 1;
        if(min < 0) min = 0;
        
        for(m = min; m < s1 && m <= n; ++m)
            out[n] += f[m].y * g[n-m];
    }
}

int main(int argc, char* argv[])
{
    /* read program options */
    char plot = 0;
    double limit;
    if(argc >= 2) plot  = argv[1][0] == 'p';
    if(argc >= 3) limit = atof(argv[2]); else limit = 0.4;
    
    /* read the input data */
    Data data;
    data.read(limit);
    
    double* out = new double[data.size() + n_coef];
    
    /* filters the data */
    conv(data, coef, n_coef, out);
    
    /* iterates the data to find the maximum */
    Pair peak = {-1, -1}, cur;
    int d = (n_coef - 1) / 2;
    
    for(size_t i = 0, e = data.size(); i < e; ++i)
    {
        cur.x = data[i].x;
        cur.y = out[i + d];
        peak.cmp_peak(cur);
        if(plot) printf("%.12lf %.12lf\n", cur.x, cur.y);
    }
    
    if(!plot) printf("%lf\n", peak.x);
    delete[] out;
    
    return 0;
}
