/**
 * Tries to fit the input spectrum with a gaussian. The gaussian
 * parameters are optimized with a least squares algorithm (Gauss-Newton).
 * 
 * Usage:
 * 
 * ./ls_gaussian plot limit < input_spectrum 
 */

#include <armadillo>
#include "common.cpp"

/**
 * Gaussian.
 * 
 * @param x Data point
 * @param a Amplitude
 * @param c Center
 * @param d Standard deviation
 */
double gaussian(double x, double a, double c, double d)
{
    double t = (c - x);
    return a * exp( -t*t / (2*d*d) );
}

/* Derivatives */
 
double gaussian_da(double x, double a, double c, double d)
{
    double t = (c - x);
    return exp( -t*t / (2*d*d) );
}

double gaussian_dc(double x, double a, double c, double d)
{
    double t = (x - c);
    return (a * t) / (d*d) * exp(-t*t / (2*d*d));
}

double gaussian_dd(double x, double a, double c, double d)
{
    double t = (x - c);
    return a * t * t / (d*d*d) * exp(-t*t / (2*d*d));
}

int main(int argc, char* argv[])
{
    /* read program options */
    char plot = 0;
    double limit;
    if(argc >= 2) plot  = argv[1][0] == 'p';
    if(argc >= 3) limit = atof(argv[2]); else limit = 0.4;
    
    /* read and scale the input */
    Data data;
    data.read(limit);
    
    double min = data.min_x, max = data.max_x;
    data.scale_x(0, 1);
    
    arma::mat jacobian( data.size(), 3);
    arma::vec error( data.size() );
    arma::vec p(3); /* params: amplitude, center and standard deviation */
    
    /* initial parameters */
    p(0) = 1;
    p(1) = 0.5;
    p(2) = 0.3;
    
    for(int it = 0; it < 5; ++it)
    {
        /* build the jacobian and error matrices */
        for(size_t n = 0, e = data.size(); n < e; ++n)
        {
            jacobian(n, 0) = gaussian_da( data[n].x, p(0), p(1), p(2) );
            jacobian(n, 1) = gaussian_dc( data[n].x, p(0), p(1), p(2) );
            jacobian(n, 2) = gaussian_dd( data[n].x, p(0), p(1), p(2) );
            error(n)       = data[n].y - gaussian( data[n].x, p(0), p(1), p(2) );
        }
        
        /* compute the parameter changes */
        arma::mat hessian  = trans(jacobian) * jacobian;
        arma::mat gradient = trans(jacobian) * error;
        p += solve(hessian, gradient);
    }
    
    /* find the peak using the maximum algorithm */
    double step_size = 1e-4;
    int n_steps      = (max - min) / step_size;
    Pair peak = {-1, -1}, cur;
    
    for(int it = 0; it <= n_steps; ++it)
    {
        cur.x    = min + it * step_size; 
        double s = scale(cur.x, min, max, 0, 1);
        cur.y    = gaussian(s, p(0), p(1), p(2) );
        
        peak.cmp_peak(cur);
        if(plot) printf("%.12lf %.12lf\n", cur.x, cur.y);
    }
    
    if(!plot) printf("%lf\n", peak.x);
    
    return 0;
}
