/**
 * Tries to fit the input spectrum with a polinomial. The polinomial coefficients
 * are optimized with one step of the Gauss-Newton method.
 * 
 * Usage:
 * 
 * ./ls_gaussian plot limit < input_spectrum 
 */

#include <armadillo>
#include <cmath>
#include "common.cpp"

#define N_COEF 4

double poly(double x, arma::vec& coef)
{
    int n = coef.n_rows;
    double res = 0;
    
    for(int i = 0; i < n; ++i)
        res += coef[i] * pow(x, i);
    
    return res;
}

double diff(double x, double i)
{
    return pow(x, i);
}

int main(int argc, char* argv[])
{
    /* read program options */
    char plot = 0;
    double limit;
    if(argc >= 2) plot  = argv[1][0] == 'p';
    if(argc >= 3) limit = atof(argv[2]); else limit = 0.8;
    
    /* read and scale the input */
    Data data;
    data.read(limit);
    
    double min = data.min_x, max = data.max_x;
    data.scale_x(0, 1);
    
    arma::mat jacobian( data.size(), N_COEF);
    arma::vec error( data.size() );
    arma::vec c(N_COEF); /* params: c0, c1, c2, c3 */
    
    /* initial coefficients */
    for(int i = 0; i < N_COEF; ++i)
        c(i) = 0.5;
    
    /* build the jacobian and error matrices */
    for(size_t n = 0, e = data.size(); n < e; ++n)
    {
        double x = data[n].x;
        
        for(int i = 0; i < N_COEF; ++i)
            jacobian(n, i) = diff(x, i);
            
        error(n) = data[n].y - poly(x, c);
    }
        
    /* compute the coefficient changes */
    arma::mat hessian  = trans(jacobian) * jacobian;
    arma::mat gradient = trans(jacobian) * error;
    c += solve(hessian, gradient);
    
    /* find the peak using the maximum algorithm */
    double step_size = 1e-4;
    int n_steps      = (max - min) / step_size;
    Pair peak = {-1, -1};
    
    for(int it = 0; it <= n_steps; ++it)
    {
        Pair cur;
        cur.x    = min + it * step_size; 
        double s = scale(cur.x, min, max, 0, 1);
        cur.y    = poly(s, c);
        
        peak.cmp_peak(cur);
        if(plot) printf("%.12lf %.12lf\n", cur.x, cur.y);
    }
    
    if(!plot) printf("%lf\n", peak.x);
    
    return 0;
}
