/**
 * Adds noise to a spectrum. 
 * Since we can't predict the exact SNR, we generate random noise until we
 * get a SNR close to the wanted value.
 * 
 * Usage:
 * 
 * ./noise_add noise_level wanted_snr limit < input_spectrum
 */

#include "common.cpp"

int main(int argc, char* argv[])
{
    double limit;
    
    if(argc < 3)
    {
        fprintf(stderr, "Usage: %s noise_level wanted_snr_db\n", argv[0]);
        return 0;
    }
    
    /* reads the input options */
    double level         = atof(argv[1]);
    double wanted_snr_db = atof(argv[2]);
    limit = argc > 3 ? atof(argv[3]) : 0.05;  
    
    /* reads the input data */
    Data data, aux_data;
    data.read(limit);
    aux_data.resize(data.size());
    
    srand( random_seed() );
    double signal_power = 0, noise_power, snr, snr_db;
    
    /* calculates the original signal power */
    for(size_t i = 0, e = data.size(); i < e; ++i)
        signal_power += data[i].y * data[i].y;
        
    signal_power /= data.size();
    
    /* generate a new data set, with noise, until it's signal power is close enough to
     * the desired noise */
    for(;;)
    {
        noise_power = 0;
        
        for(size_t i = 0, e = data.size(); i < e; ++i)
        {
            aux_data[i]    = data[i];
            double noise   = rand_double(level);
            noise_power   += noise * noise;
            aux_data[i].y += noise;
        }
        
        noise_power  /= data.size();
        snr           = signal_power / noise_power;
        snr_db        = 10 * log10(snr);
        fprintf(stderr, "calc: %lf\ttarget: %lf\n", snr_db, wanted_snr_db);
        if(  fabs(wanted_snr_db - snr_db) < 0.02 || wanted_snr_db == 0 ) break;
    }
    
    /* data information */
    fprintf(stderr, "--------------------\n");
    fprintf(stderr, "Signal power: %lf\n", signal_power);
    fprintf(stderr, "Noise power : %lf\n", noise_power);
    fprintf(stderr, "--------------------\n");
    fprintf(stderr, "SNR         : %lf\n", snr);
    fprintf(stderr, "SNR (dB)    : %lf\n", snr_db);
    fprintf(stderr, "--------------------\n");
    
    /* new data */
    aux_data.calc_bounds();
    aux_data.scale_y(0, 1);
    aux_data.print();
    
    return 0;
}
